/*  cfdg-fe, a Linux/GTK+ front-end for the Context Free Design Grammar
    Copyright (C) 2006  Csaba Skrabák

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"

/*info that a file-saving callback function needs*/
struct save_info{
	GtkFileSelection*fs; //for the file name selected by user
	GtkWindow*window1; //for the window title and to look up the text
};

/*new*/
void on___j1_activate(GtkMenuItem*menuitem,gpointer user_data){
	system("cfdg-fe&");
}

/*open (show dialog)*/
void on_megnyit__s1_activate(GtkMenuItem*menuitem,gpointer user_data){
	GtkWidget*w=create_opencfdgfilechooserdialog();
	gtk_widget_show(w);
}

/*open (go ahead when dialog closed)*/
void on_opencfdgfilechooserdialog_response(
	GtkDialog*dialog,
	gint response_id,
	gpointer user_data
){
	if(response_id!=GTK_RESPONSE_OK)
		return;

	gchar*fn;
	fn=gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
	char*cmd;
	cmd=(char*)malloc(strlen(fn)+10);
	sprintf(cmd,"cfdg-fe %s&",fn);
	system(cmd);
	free(cmd);
}

/*save*/
void on_ment__s1_activate(GtkWindow*window1){
	GtkTextBuffer*doc;
	GtkTextIter start_iter;
	GtkTextIter end_iter;
	gchar*contents;
	gsize len;
	ssize_t written;
	gchar*fn;
	gchar*title;

	/*title of the window containing whether a file has been saved*/
	title=gtk_window_get_title(window1);
	if(strlen(title)>14){ //if already saved and has a name
		fn=title+15; //the file name it was saved last

		/*text buffer containing everything to be written out*/
		doc=gtk_text_view_get_buffer(
			GTK_TEXT_VIEW(
				lookup_widget(
					GTK_WIDGET(window1),
					"textview1"
				)
			)
		);

		/*exploiting the contents to write*/
		gtk_text_buffer_get_bounds(doc,&start_iter,&end_iter);
		contents=gtk_text_buffer_get_slice(
			doc,&start_iter,&end_iter,TRUE
		);
		len=strlen(contents);

		/*do write it to the file*/
		int ofd=open(fn,O_WRONLY|O_TRUNC|O_CREAT,0644);
		if(ofd>=0){
			title[13]='-';
			gtk_text_buffer_set_modified(doc,FALSE);
			gtk_window_set_title(window1,title);

			write(ofd,contents,len);

			close(ofd);
		}

		/*release memory*/
		g_free(contents);
	}else //if not yet saved, redirect to save as...
		on_ment__s_m__sk__nt1_activate(window1);
}

/*save as (go ahead when dialog closed)*/
static void file_ok_sel(
	GtkWidget*okbutton,
	struct save_info*si
){
	GtkTextBuffer*doc;
	GtkTextIter start_iter;
	GtkTextIter end_iter;
	gchar*contents;
	gsize len;
	ssize_t written;
	gchar*fn=gtk_file_selection_get_filename(GTK_FILE_SELECTION(si->fs));

	/*text buffer containing everything to be written out*/
	doc=gtk_text_view_get_buffer(
		GTK_TEXT_VIEW(
			lookup_widget(
				GTK_WIDGET(si->window1),
				"textview1"
			)
		)
	);

	/*exploiting the contents to write*/
	gtk_text_buffer_get_bounds(doc,&start_iter,&end_iter);
	contents=gtk_text_buffer_get_slice(doc,&start_iter,&end_iter,TRUE);
	len=strlen(contents);

	/*write*/
	int ofd=open(fn,O_WRONLY|O_TRUNC|O_CREAT,0644);
	if(ofd>=0){
		char*title=(char*)malloc(strlen(fn)+16);
		strcpy(title,"Context Free - ");
		strcat(title,fn);
		gtk_window_set_title(si->window1,(gchar*)title);
		gtk_text_buffer_set_modified(doc,FALSE);

		write(ofd,contents,len);

		close(ofd);
	}

	/*tidy memory*/
	g_free(contents);
	gtk_widget_destroy(GTK_WIDGET(si->fs));
	free(si);
}

/*save as (open dialog)*/
void on_ment__s_m__sk__nt1_activate(GtkWindow*window1){
	GtkWidget *filew;

	/* Create a new file selection widget */
	filew = gtk_file_selection_new("Save CFDG Source As...");

	struct save_info*si=(struct save_info*)malloc(sizeof(struct save_info));
	si->window1=window1;
	si->fs=GTK_FILE_SELECTION(filew);

	/* Connect the ok_button to file_ok_sel function */
	g_signal_connect(
		G_OBJECT(GTK_FILE_SELECTION(filew)->ok_button),
		"clicked",
		G_CALLBACK(file_ok_sel),
		(gpointer)si
	);

	/* Connect the cancel_button to destroy the widget */
	g_signal_connect_swapped(
		G_OBJECT(
			GTK_FILE_SELECTION(filew)->cancel_button
		),
		"clicked",
		G_CALLBACK(gtk_widget_destroy),
		G_OBJECT(filew)
	);

	gtk_window_set_modal(GTK_WINDOW(filew),TRUE);
	gtk_window_set_destroy_with_parent(GTK_WINDOW(filew),TRUE);
	gtk_window_set_type_hint(GTK_WINDOW(filew),GDK_WINDOW_TYPE_HINT_DIALOG);
	gtk_widget_show(filew);
}

/*cut*/
void
on_kiv__g__s1_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}

/*copy*/
void
on_m__sol__s1_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}

/*paste*/
void
on_beilleszt__s1_activate              (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}

/*delete*/
void
on_t__rl__s1_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}

/*about*/
void on_n__vjegy1_activate(GtkMenuItem*menuitem,gpointer user_data){
	GtkWidget*w=create_aboutdialog();
	gtk_widget_show(w);
}

/*render to size dialog: enable/disable max shapes spin button*/
void on_maxshapescheckbutton_toggled(GtkDialog*dialog){
	gtk_widget_set_sensitive(
		lookup_widget(
			GTK_WIDGET(dialog),
			"maxshapesspinbutton"
		),
		gtk_toggle_button_get_active(
			GTK_TOGGLE_BUTTON(
				lookup_widget(
					GTK_WIDGET(dialog),
					"maxshapescheckbutton"
				)
			)
		)
	);
}

/*mark as unsaved*/
void on_textview1_changed(GtkTextView*textview,gpointer user_data){
	GtkWindow*window1=GTK_WINDOW(gtk_widget_get_toplevel(GTK_WIDGET(textview)));

	gchar*title=gtk_window_get_title(window1);
	if(strlen(title)>14)title[13]='*';
	gtk_window_set_title(window1,title);
}

/*mark as unsaved, alternative function header*/
gboolean on_textview1_changed_(
	GtkWidget*widget,
	GdkEventKey*event,
	gpointer user_data
){
	on_textview1_changed((GtkTextView*)widget,user_data);
	return FALSE;
}

/*view/render to size (go ahead when dialog closed)*/
static void render_to_size_ok(
	GtkWidget*dialog,
	int response_id,
	GtkWindow*window1
){
	char*tfn;
	GtkTextBuffer*doc;
	GtkTextIter start_iter;
	GtkTextIter end_iter;
	gchar*contents;
	gsize len;
	ssize_t written;
	gchar*title;
	GtkImage*renderimage;
	gboolean
		par_m_set, //max num of shapes<infinity
		par_b_set, //whether any border size selected
		par_a_set; //animation of more than 1 frames
		
	gint par_b; //border size
	gdouble
		par_x; //minimum size of shapes
	gchar
		*seed, //-v parameter
		*par_w, //width
		*par_h, //height
		*par_m, //max num of shapes
		*par_a, //num of animation frames
		*par_z, //zoom out during animation
		*par_V, //svg output
		*par_c; //crop image output
	char*cmd=(char*)malloc(1024);
	char*png=(char*)malloc(1024);
	
	if(response_id!=GTK_RESPONSE_OK)
		return;

	/*auto increment seed if not manually set*/
	if(
		gtk_toggle_tool_button_get_active(
			GTK_TOGGLE_TOOL_BUTTON(
				lookup_widget(
					GTK_WIDGET(window1),
					"seedused"
				)
			)
		)==TRUE
	)
		on_seedincbutton_clicked(
			GTK_ENTRY(
				lookup_widget(
					GTK_WIDGET(window1),
					"seedentry"
				)
			)
		);

	/*get the seed*/
	seed=gtk_entry_get_text(
		GTK_ENTRY(
			lookup_widget(
				GTK_WIDGET(window1),
				"seedentry"
			)
		)
	);

	/*get params from the dialog*/
	par_w=gtk_entry_get_text(
		lookup_widget(
			dialog,
			"widthspinbutton"
		)
	);
	par_h=gtk_entry_get_text(
		lookup_widget(
			dialog,
			"heightspinbutton"
		)
	);
	par_m_set=gtk_toggle_button_get_active(
		lookup_widget(
			dialog,
			"maxshapescheckbutton"
		)
	);
	par_m=gtk_entry_get_text(
		lookup_widget(
			dialog,
			"maxshapesspinbutton"
		)
	);
	par_x=gtk_spin_button_get_value(
		lookup_widget(
			dialog,
			"minshapesizespinbutton"
		)
	);
	par_b=gtk_combo_box_get_active(
		lookup_widget(
			dialog,
			"bordercombobox"
		)
	)-1;
	par_b_set=(par_b>-2);
	par_a=gtk_entry_get_text(
		lookup_widget(
			dialog,
			"numframesspinbutton"
		)
	);
	par_a_set=(atoi(par_a)>1);
	par_z=gtk_toggle_button_get_active(
		lookup_widget(
			dialog,
			"maxshapescheckbutton"
		)
	)?" -z":"";
	par_V=(gtk_combo_box_get_active(
		lookup_widget(
			dialog,
			"formatcombobox"
		)
	)==1?" -V":"");
	par_c=(gtk_toggle_button_get_active(
		lookup_widget(
			dialog,
			"cropcheckbutton"
		)
	)==TRUE?" -c":"");

	/*text buffer containing everything to be written out
	(technique is that we write current source to a temp file
	then pass that temp file over to the cfdg compiler)*/
	doc=gtk_text_view_get_buffer(
		GTK_TEXT_VIEW(
			lookup_widget(
				GTK_WIDGET(window1),
				"textview1"
			)
		)
	);

	/*exploiting the contents to write*/
	gtk_text_buffer_get_bounds(doc,&start_iter,&end_iter);
	contents=gtk_text_buffer_get_slice(doc,&start_iter,&end_iter,TRUE);
	len=strlen(contents);

	/*temp file generation*/
	tfn=(char*)malloc(1024);
	strncpy(tfn,"/tmp/cfdg-fe-source-XXXXXX",1024);
	int tfd=mkstemp(tfn);

	/*render*/
	if(tfd>=0){
		/*write to file*/
		write(tfd,contents,len);
		close(tfd);

		/*assemble the command to execute*/
		snprintf(png,1024,"/tmp/cfdg-fe-preview-%d.png",getpid());
		int i=snprintf(
			cmd,
			1024,
			"cfdg -v %s -w %s -h %s%s%s%s -x %d.%d",
			seed, //-v parameter
			par_w, //width
			par_h, //height
			par_z, //zoom out during animation
			par_V, //svg output
			par_c, //crop image output
			((int)(par_x*1000))/1000, //minimum size of shapes, integers
			((int)(par_x*1000))%1000 //minimum size of shapes, decimals
		);
		if(par_m_set)i+=snprintf(
			cmd+i,
			1024-i,
			" -m %s",
			par_m //max num of shapes
		);
		if(par_b_set)i+=snprintf(
			cmd+i,
			1024-i,
			" -b %d",
			par_b //border size
		);
		if(par_a_set)i+=snprintf(
			cmd+i,
			1024-i,
			" -a %s",
			par_a //num of animation frames
		);
		snprintf(
			cmd+i,
			1024-i,
			" %s %s >/tmp/cfdg-fe-render-%d.log",
			tfn, //temporary file name (cfdg source)
			png, //output image (png/svg) file name
			getpid() //pid
		);

		/*do execute it*/
		system(cmd);

		/*delete temp file*/
		unlink(tfn);

		/*update displayed image*/
		renderimage=GTK_IMAGE(lookup_widget(GTK_WIDGET(window1),"renderimage"));
		if(par_a_set){ //when animation made
			char*frm=(char*)malloc(1024);
			int maxframe=atoi(par_a);
			snprintf(frm,1024,"/tmp/cfdg-fe-preview-%d_%d.png",getpid(),maxframe);
			gtk_image_set_from_file(renderimage,frm);
			free(frm);
			GtkWidget*framescale=lookup_widget(window1,"framescale");
			gtk_range_set_range(
				GTK_RANGE(framescale),
				0,
				(gdouble)maxframe
			);
			gtk_range_set_value(
				GTK_RANGE(framescale),
				(gdouble)maxframe
			);
			gtk_widget_show(framescale);
		}else{
			gtk_image_set_from_file(renderimage,png);
			gtk_widget_hide(lookup_widget(window1,"framescale"));
		}
	}else{ //open failed
		perror("cannot write to temp file when rendering");
	}

	/*mark seed as used*/
	gtk_toggle_tool_button_set_active(
		GTK_TOGGLE_TOOL_BUTTON(
			lookup_widget(
				GTK_WIDGET(window1),
				"seedused"
			)
		),
		TRUE
	);

	/*clean-up*/
	free(tfn);

	/*display text output of the renderer*/
	char*ofn=(char*)malloc(1024);
	snprintf(ofn,1024,"/tmp/cfdg-fe-render-%d.log",getpid());
	int ofd=-1;
	ofd=open(tfn,O_RDONLY);
	struct stat ostat;
	if(ofd>=0){
		fstat(ofd,&ostat);
		char*filecontent=(char*)malloc(strlen(cmd)+ostat.st_size+2);
		strcpy(filecontent,cmd);
		strcat(filecontent,"\n");
		read(ofd,filecontent+strlen(filecontent),ostat.st_size);
		filecontent[strlen(cmd)+ostat.st_size+1]=0;
		gint len=(gint)strlen(filecontent);

		close(ofd);

		gtk_text_buffer_set_text(
			gtk_text_view_get_buffer(
				GTK_TEXT_VIEW(
					lookup_widget(
						GTK_WIDGET(window1),
						"textview2"
					)
				)
			),
			filecontent,
			len
		);

		/*memory clean-up*/
		free(filecontent);
	}
	free(png);
	free(cmd);
}

/*view/render to size (open dialog)*/
void on_render_to_size1_activate(GtkWindow*window1){
	GtkWidget*w=create_rendertosizedialog();

	/*connection not done in create function because
	callback needs window pointer
	(glade does not support passing it over)*/
	g_signal_connect(
		(gpointer)w,
		"response",
		G_CALLBACK(render_to_size_ok),
		(gpointer)window1
	);

	gtk_widget_show(w);
}

/*call cfdg compiler with default parameters*/
void on_renderbutton_clicked(GtkWindow*window1){
	char*tfn;
	GtkTextBuffer*doc;
	GtkTextIter start_iter;
	GtkTextIter end_iter;
	gchar*contents;
	gsize len;
	ssize_t written;
	gchar*title;
	GtkImage*renderimage;
	gchar*seed;
	char*cmd=(char*)malloc(1024);
	char*png=(char*)malloc(1024);
	
	/*auto increment seed if not manually set*/
	if(
		gtk_toggle_tool_button_get_active(
			GTK_TOGGLE_TOOL_BUTTON(
				lookup_widget(
					GTK_WIDGET(window1),
					"seedused"
				)
			)
		)==TRUE
	)
		on_seedincbutton_clicked(
			GTK_ENTRY(
				lookup_widget(
					GTK_WIDGET(window1),
					"seedentry"
				)
			)
		);

	/*get the seed*/
	seed=gtk_entry_get_text(
		GTK_ENTRY(
			lookup_widget(
				GTK_WIDGET(window1),
				"seedentry"
			)
		)
	);

	/*text buffer containing everything to be written out*/
	doc=gtk_text_view_get_buffer(
		GTK_TEXT_VIEW(
			lookup_widget(
				GTK_WIDGET(window1),
				"textview1"
			)
		)
	);

	/*exploiting the contents to write*/
	gtk_text_buffer_get_bounds(doc,&start_iter,&end_iter);
	contents=gtk_text_buffer_get_slice(doc,&start_iter,&end_iter,TRUE);
	len=strlen(contents);

	/*temp file generation*/
	tfn=(char*)malloc(1024);
	strncpy(tfn,"/tmp/cfdg-fe-source-XXXXXX",1024);
	int tfd=mkstemp(tfn);

	if(tfd>=0){
		write(tfd,contents,len);
		close(tfd);
		snprintf(png,1024,"/tmp/cfdg-fe-preview-%d.png",getpid());
		snprintf(cmd,1024,"cfdg -s 384 -m 300000 -x 0.4 -v %s %s %s >/tmp/cfdg-fe-render-%d.log",seed,tfn,png,getpid());
		system(cmd);
		unlink(tfn);
		renderimage=GTK_IMAGE(lookup_widget(GTK_WIDGET(window1),"renderimage"));
		gtk_image_set_from_file(renderimage,png);
	}else{
		perror("cannot write to temp file when rendering");
	}
	
	gtk_toggle_tool_button_set_active(
		GTK_TOGGLE_TOOL_BUTTON(
			lookup_widget(
				GTK_WIDGET(window1),
				"seedused"
			)
		),
		TRUE
	);

	free(tfn);

	/*display text output of the renderer*/
	char*ofn=(char*)malloc(1024);
	snprintf(ofn,1024,"/tmp/cfdg-fe-render-%d.log",getpid());
	int ofd=-1;
	ofd=open(tfn,O_RDONLY);
	struct stat ostat;
	if(ofd>=0){
		fstat(ofd,&ostat);
		char*filecontent=(char*)malloc(strlen(cmd)+ostat.st_size+2);
		strcpy(filecontent,cmd);
		strcat(filecontent,"\n");
		read(ofd,filecontent+strlen(filecontent),ostat.st_size);
		filecontent[strlen(cmd)+ostat.st_size+1]=0;
		gint len=(gint)strlen(filecontent);

		close(ofd);

		gtk_text_buffer_set_text(
			gtk_text_view_get_buffer(
				GTK_TEXT_VIEW(
					lookup_widget(
						GTK_WIDGET(window1),
						"textview2"
					)
				)
			),
			filecontent,
			len
		);

		free(filecontent);
	}
	free(png);
	free(cmd);
	gtk_widget_hide(lookup_widget(window1,"framescale"));
}


/*save rendered picture (go ahead when dialog closed)*/
static void pngfile_ok_sel(
	GtkWidget*okbutton,
	gpointer user_data
){
	GtkWidget*dialog=gtk_widget_get_toplevel(okbutton);
	gchar*fn=gtk_file_selection_get_filename(
		GTK_FILE_SELECTION(dialog)
	);

	char*cmd=(char*)malloc(1024);
	snprintf(cmd,1024,"cp /tmp/cfdg-fe-preview-%d.png %s",getpid(),fn);
	system(cmd);
	free(cmd);
	gtk_widget_destroy(dialog);
}

/*save rendered picture (open file chooser dialog)*/
void on_savepicbutton_clicked(GtkToolButton*toolbutton,gpointer user_data){
	GtkWidget*filew;

	/* Create a new file selection widget */
	filew=gtk_file_selection_new("Save Image As...");

	/* Connect the ok_button to file_ok_sel function */
	g_signal_connect(
		G_OBJECT(GTK_FILE_SELECTION(filew)->ok_button),
		"clicked",
		G_CALLBACK(pngfile_ok_sel),
		(gpointer)NULL
	);

	/* Connect the cancel_button to destroy the widget */
	g_signal_connect_swapped(
		G_OBJECT(GTK_FILE_SELECTION(filew)->cancel_button),
		"clicked",
		G_CALLBACK(gtk_widget_destroy),
		G_OBJECT(filew)
	);

	gtk_window_set_modal(GTK_WINDOW(filew),TRUE);
	gtk_window_set_destroy_with_parent(GTK_WINDOW(filew),TRUE);
	gtk_window_set_type_hint(GTK_WINDOW(filew),GDK_WINDOW_TYPE_HINT_DIALOG);
	gtk_widget_show(filew);
}

/*decrement random seed*/
void on_seeddecbutton_clicked(GtkEntry*entry){
	int i;
	int numeric=0;
	gchar*text=gtk_entry_get_text(entry);

	for(i=0;i<3;i++){
		numeric=26*numeric+(int)(text[i]-'A');
	}
	numeric--;
	if(numeric<0)numeric=25+26*(25+26*25); //ZZZ
	for(i=2;i>=0;i--){
		text[i]=numeric%26+'A';
		numeric/=26;
	}
	gtk_entry_set_text(entry,text);
	gtk_widget_grab_focus(GTK_WIDGET(entry));
	gtk_entry_select_region(entry,3,3);
	
	gtk_toggle_tool_button_set_active(
		GTK_TOGGLE_TOOL_BUTTON(
			lookup_widget(
				gtk_widget_get_parent(
					gtk_widget_get_parent(
						GTK_WIDGET(entry)
					)
				),
				"seedused"
			)
		),
		FALSE
	);
}

/*increment random seed*/
void on_seedincbutton_clicked(GtkEntry*entry){
	int i;
	int numeric=0;
	gchar*text=gtk_entry_get_text(entry);

	for(i=0;i<3;i++){
		numeric=26*numeric+(int)(text[i]-'A');
	}
	numeric++;
	if(numeric>25+(26*(25+26*25)))numeric=0;
	for(i=2;i>=0;i--){
		text[i]=numeric%26+'A';
		numeric/=26;
	}
	gtk_entry_set_text(entry,text);
	gtk_widget_grab_focus(GTK_WIDGET(entry));
	gtk_entry_select_region(entry,3,3);
	
	gtk_toggle_tool_button_set_active(
		GTK_TOGGLE_TOOL_BUTTON(
			lookup_widget(
				gtk_widget_get_parent(
					gtk_widget_get_parent(
						GTK_WIDGET(entry)
					)
				),
				"seedused"
			)
		),
		FALSE
	);
}

void on_seedentry_changed(GtkToggleToolButton*b){
	gtk_toggle_tool_button_set_active(b,FALSE);
}

/*display copyright details*/
void on_copyrightdetails_activate(GtkMenuItem*menuitem,gpointer user_data){
	GtkDialog*dialog=gtk_message_dialog_new(
		GTK_WINDOW(
			gtk_widget_get_toplevel(
				GTK_WIDGET(menuitem)
			)
		),
		GTK_DIALOG_DESTROY_WITH_PARENT,
		GTK_MESSAGE_INFO,
		GTK_BUTTONS_CLOSE,
		"Please do me no harm."
	);
	gtk_dialog_run(dialog);
	gtk_widget_destroy(GTK_WIDGET(dialog));
}

/*display redistribution conditions*/
void on_redistribution_conditions1_activate(
	GtkMenuItem*menuitem,
	gpointer user_data
){
	GtkDialog*dialog=gtk_message_dialog_new(
		GTK_WINDOW(
			gtk_widget_get_toplevel(
				GTK_WIDGET(menuitem)
			)
		),
		GTK_DIALOG_DESTROY_WITH_PARENT,
		GTK_MESSAGE_INFO,
		GTK_BUTTONS_CLOSE,
		"Please do me no harm."
	);
	gtk_dialog_run(dialog);
	gtk_widget_destroy(GTK_WIDGET(dialog));
}

/*safely close main window*/
void on_window1_destroy(GtkObject*object,gpointer user_data){
	/*get the source text*/
	GtkTextBuffer*buf=gtk_text_view_get_buffer(
		GTK_TEXT_VIEW(
			lookup_widget(
				GTK_WIDGET(object),
				"textview1"
			)
		)
	);

	/*ask user about saving if source modified*/
	if(gtk_text_buffer_get_modified(buf)==TRUE){
		GtkDialog*dialog=GTK_DIALOG(
			gtk_message_dialog_new(
				GTK_WINDOW(object),
				GTK_DIALOG_DESTROY_WITH_PARENT,
				GTK_MESSAGE_QUESTION,
				GTK_BUTTONS_YES_NO,
				"Save before closing?"
			)
		);
		gint result=gtk_dialog_run(dialog);
		if(result==GTK_RESPONSE_YES)
			on_ment__s1_activate(GTK_WINDOW(object));
		gtk_widget_destroy(GTK_WIDGET(dialog));
	}

	/*delete temporary files*/
	char*cmd=(char*)malloc(1024);
	snprintf(cmd,1024,"rm /tmp/cfdg-fe-*-%d.* /tmp/cfdg-fe-preview-%d_*.* 2>/dev/null",getpid(),getpid());
	system(cmd);
	free(cmd);
}


void on_framescale_move_slider(
	GtkRange*range,
	GtkScrollType scroll,
	gpointer user_data
){
	int val=gtk_range_get_value(range);
	int logmax=(int)log10(gtk_range_get_adjustment(range)->upper)+1;
	char*frm=(char*)malloc(1024);
	snprintf(frm,1024,"/tmp/cfdg-fe-preview-%d_%0*d.png",getpid(),logmax,val);
	gtk_image_set_from_file(
		GTK_IMAGE(
			lookup_widget(
				gtk_widget_get_toplevel(range),
				"renderimage"
			)
		),
		frm
	);
	free(frm);
}
